
# The database connection string must be sqlalchemy compatible
DB_CONNECTION_STRING = "sqlite:///spynepi.db"

# The path(absolute recommended) where package files will be saved
FILES_PATH = 'files'

# The server address
HOST = "0.0.0.0"

# The server port
PORT = 7789

# The pypi repository name which defined in pypirc
REPO_NAME = "spynepi"

# The prefix for database tables
TABLE_PREFIX = "spynepi"
